// FILE: cluster1_0a.c - Compiles but can't figure out how to display number of objects
// DATE: 02/11/20 13:39
// AUTH: P.Oh
// DESC: Clustering: count number of objects in image
// VERS: 1.0a: Pocket Handbook pg. 44-45
// REFS: negative1_1.c

#include<stdlib.h>
#include<stdio.h>
#include<memory.h>
#include<math.h>

#include "image.h"

#define pix(Im, x, y)   *(Im->Data + (x)*Im->Cols + (y))
#define WHITE           255 // pg. 94
#define BLACK           0

int area(struct Image *In, int x1, int y1,
                int x2, int y2, unsigned char ObjVal) {

   long i, j;
   int area_value = 0;

   for(i=x1; i <= x2; ++i)
     for(j=y1; j <= y2; ++j) {
        if(pix(In, i, j)==ObjVal) ++area_value;
     }
    return(area_value);

} // end function area

void Img_in(struct Image *Img) {
    FILE *ifile;
    int i;

    // NB: Assumes already know that using a RAW image file 256 x 256 size
    // Open file for binary reading
    ifile = fopen("20x28-clusterWhiteObjectsBlackBackgroundRaw.raw", "rb"); // rb: reading text or binary files

    // Read directly into the image array
    for(i=0; i < Img->Rows; ++i)
        fread(Img->Data + i*Img->Cols, Img->Cols, 1, ifile);

    fclose(ifile);

} // end Img_in

void Img_process(struct Image *In, struct Image *Out) {

 long i, j;
 int val, thresholdValue;
 unsigned char *tmp = NULL;

 thresholdValue = 128;

 for(i=0; i<In->Rows; ++i) {
    for(j=0; j<In->Cols; ++j) {
        val = *(Out->Data + i*Out->Rows + j);
        if(val < thresholdValue) {
            val = 0;
        }
        else {
            val = 255;
        }
        *tmp = val;
    };

 };

}

struct coord {
  float x, y;
};

void *centroid(struct Image *In, int x1,
               int y1, int x2, int y2,
	       unsigned char ObjVal,
               struct coord *coords) {
  long i, j;
  int area_value, Xcent = 0, Ycent = 0;

  area_value = area(In, x1, y1, x2, y2, ObjVal);

  if(area_value == 0) {
    coords->x = -1; coords->y = -1;
    return 0;
  };

  for(i=x1; i<=x2; ++i)
    for(j=y1; j<=y2; ++j) {
        if(pix(In, i, j) == ObjVal) {
            Xcent += j;
            Ycent += i;
        }
    }

   coords->x = Xcent/area_value + 1;
   coords->y = Ycent/area_value + 1;

   return 0;
} // end function centroid

void Img_out(struct Image *Out) {
    FILE *ofile;
    int i;

    // open (or create) a file for writing
    // ofile = fopen("OUTPUT.RAW", "wb");
    ofile = fopen("OUTPUT.RAW", "wb");
    // out the image by rows
    for(i=0; i < Out->Rows; ++i)
        fwrite(Out->Data + i*Out->Cols, Out->Cols, 1, ofile);

    fclose(ofile);
} // end Img_out

cluster(struct Image *In, int clust_size) {
  int n, i, j, k, new_clust;
  float edist, clustx[256], clusty[256];
  int numberOfObjects = 0;
  // initialize cluster count variable n
  n = 0;
  // double-loop thru each pixel of binary image
  for(i=0; i < In->Rows; ++i) {
    for(j=0; j < In->Cols; ++j) {
        if(n==0) {
            // only do for 1st cluster
            if(pix(In, i, j) != WHITE) {
                    n=1; // 1st cluster found
                    // store X and Y coordinates and mark pixel as cluster 1
                    clustx[1] = i;
                    clusty[1] = j;
                    pix(In, i, j) = 1;
            }
        }
        // test for membership in all known clusters
        else if(pix(In, i, j) != WHITE) {
            // marker for new cluster
            new_clust = 0;
            // compute Euclidean distance
            for(k=1; k <= n; ++k) {
                edist = sqrt( (i-clustx[k])*(i-clustx[k]) + (j-clusty[k])*(j-clusty[k]) );
                // test against cluster size desired
                if(edist <= clust_size) {
                    // set pixel to cluster number
                    pix(In, i, j) = (unsigned char)k;
                    new_clust = 1;
                    k = n+1;
                    numberOfObjects++;
                }; // end if
            }; // end for loop k
        }; // end else if
    }; // end for loop j
  }; // end for loop i

  // add a new cluster
  if(new_clust == 0 && n <=255) {
      n = n+1;
      clustx[n] = i;
      clusty[n] = j;
      pix(In, i, j) = (unsigned char)n;
   };

   printf("clust[x] = %3.3f, clust[y] = %3.3f\n", clustx[n], clusty[n]);
   printf("n= %d\n", n);
   printf("k = %d\n", k);
   printf("numberOfObjects = %d\n", numberOfObjects);

} // end cluster function
int main() {

  FILE *ofile;
  struct Image In, Out; // Declare input and output images
  struct coord *centroidCoordinates;
  int clusterSize = 1;
  // Indexes for mask generation section
  // signed char *tmp = NULL;
  int areaImage;

  // Initialize image parameters
  In.Rows = Out.Rows = 20;
  In.Cols = Out.Cols = 28;
  // Allocate memory for image
  In.Data = (unsigned char *)calloc(In.Rows, In.Cols);

  Out.Data = (unsigned char *)calloc(Out.Rows, Out.Cols);
  ofile = fopen("OUTPUT.RAW", "wb");

  Img_in(&In);

  cluster(&In, clusterSize);
  printf("clusterSize = %d\n", clusterSize);
  /*
  areaImage = area(&In, 0, 0, (In.Rows-1), (In.Cols-1), BLACK); // 16x15 image goes from 0 to 15
  printf("Area of 16x16 image is: %d\n", areaImage);
  centroid(&In, 0, 0, (In.Rows-1), (In.Cols-1), BLACK, centroidCoordinates);
  printf("Centroid is (x,y) = (%3.3f, %3.3f)\n", centroidCoordinates->x, centroidCoordinates->y);
  */

  fclose(ofile);
} // end of main
